#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "condition4xi.py">
#<abstract4xi abstract = "provides Condition4Xi class"/>
#<refer4xi reference = "synchronize.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Condition4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # imported-as ones 
    from time import time as _time, sleep as _sleep          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones 
    import _multiprocessing
except:
    pass
#"
#/>
     
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from assert_spawning4xi import assert_spawning4xi
from reason_object4xi import reason_object4xi
from rlock4xi import RLock4Xi
from semaphore4xi import Semaphore4Xi
from sense_object4xi import sense_object4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
#"
#/>


        
# =================================================================================================
#<class4xi class = "Condition4Xi" base = "object" child = "None" synopsis = "Synchronizing with condition" usage = "
#    synchronize.py
#        __init__(self):
#            self._cond = Condition(Lock())
#
#    event4xi.py:
#        self._cond = Condition4Xi(Lock4Xi())
#    joinablequeue4xi.py:
#        self._cond = Condition4Xi()
#    queue4xi.py:
#        self._notempty = Condition4Xi(Lock4Xi())
#    syncmanager4xi.py:
#        SyncManager4Xi.register('Condition4Xi', threading.Condition, ConditionProxy4Xi)   "
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class Condition4Xi(object):
    '''
    Synchronizing with condition
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('wait', 'notify', 'notify_all', 'wait_for', 'sense', 'reason', 'ace')        
    #"
    #>                                     
  
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Condition4Xi" usage = "initializes Condition4Xi">                            
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, lock=None):
        '''
        initializes Condition4Xi
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)           
        # -----------------------------------------------------------------------------------------
        class2print = 'Condition4Xi' 
        method2print = '__init__'
        
        message2print = "{(Step 1/3a) started" + "; init-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        message2print = "{(Step 1/3b) passed" + "; lock-passed-to-init-of-Condition4Xi: " + str(lock) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        
        message2print = "{(Step 1/3c) borning" + "; condition2xi-of-Condition4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
   

        message2print = "{(Step 1/3d) inheriting" + "; object-inheriting-by-condition2xi-of-Condition4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
                     
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)           
        # -----------------------------------------------------------------------------------------
       
        self._lock = lock or RLock4Xi()
        
        message2print = "{(Step 2/3a) set" + "; lock-of-condition2xi-set-by-init-of-Condition4Xi to: " + str(self._lock)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
   
        self._sleeping_count = Semaphore4Xi(0)
        
        message2print = "{(Step 2/3b) set" + "; sleeping-count-of-condition2xi-set-by-init-of-Condition4Xi to: " + str(self._sleeping_count)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
  
        self._woken_count = Semaphore4Xi(0)
        
        message2print = "{(Step 2/3c) set" + "; woken-count-of-condition2xi-set-by-init-of-Condition4Xi to: " + str(self._woken_count)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
   
        self._wait_semaphore = Semaphore4Xi(0)
        
        message2print = "{(Step 2/3d) set" + "; wait-semaphore-of-condition2xi-set-by-init-of-Condition4Xi to: " + str(self._wait_semaphore)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
  
        self._make_methods()
        
        message2print = "{(Step 2/3e) performed" + "; make-methods-of-condition2xi-performed-by-init-of-Condition4Xi: " + str(self._make_methods) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                 
      
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)           
        # -----------------------------------------------------------------------------------------

        message2print = "{(Step 3/3a) activated" + "; condition2xi-of-Condition4Xi-activated: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 3/3b) inherited" + "; object-inherited-by-condition2xi-of-Condition4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
               
        message2print = "{(Step 3/3b) returning" + "; init-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                        
    #</methodize4xi>
                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getstate-of-Condition4Xi" usage = "gets state with (self)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __getstate__(self):
        '''
        Condition4Xi method __getstate__: gets state with (self)
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2)
        # -----------------------------------------------------------------------------------------        
        class2print = 'Condition4Xi'
        method2print = '__getstate__'        
        
        
        message2print = "{(Step 1/2a) started" + "; getstate-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        assert_spawning4xi(self)
        
        message2print = "{(Step 1/2b) performed" + "; assert-spawning4xi-performed-by-getstate-of-Condition4Xi: " + str(assert_spawning4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/2)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 2/2a) returning" + "; lock-of-condition2xi-returning-by-getstate-of-Condition4Xi: " + str(self._lock) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
        message2print = "{(Step 2/2b) returning" + "; sleeping-count-of-condition2xi-returning-by-getstate-of-Condition4Xi: " + str(self._sleeping_count) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 2/2c) returning" + "; woken-count-of-condition2xi-returning-by-getstate-of-Condition4Xi: " + str(self._woken_count) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 2/2d) returning" + "; wait-semaphore-of-condition2xi-returning-by-getstate-of-Condition4Xi: " + str(self._wait_semaphore) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                                
        return (self._lock, self._sleeping_count,
                self._woken_count, self._wait_semaphore)
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "setstate-of-Condition4Xi" usage = "sets state">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __setstate__(self, state):
        '''
        sets state
        '''
        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = '__setstate__'
        
        message2print = "{(Step 1/3a) started" + "; setstate-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
    
        message2print = "{(Step 1/3b) passed" + "; state-passed-to-setstate-of-Condition4Xi: " + str(state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------                        
        self._lock, self._sleeping_count,
        (self._woken_count, self._wait_semaphore) = state
        
        message2print = "{(Step 2/3a) set" + "; lock-of-condition2xi-set-by-setstate-of-Condition4Xi to: " + str(self._lock)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
        
        message2print = "{(Step 2/3b) set" + "; sleeping-count-of-condition2xi-set-by-setstate-of-Condition4Xi to: " + str(self._sleeping_count)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                              
        
        message2print = "{(Step 2/3c) set" + "; woken-count-of-condition2xi-set-by-setstate-of-Condition4Xi to: " + str(self._woken_count)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        message2print = "{(Step 2/3d) set" + "; wait-semaphore-of-condition2xi-set-by-setstate-of-Condition4Xi to: " + str(self._wait_semaphore)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                      
        try:
            self._make_methods()
        
            message2print = "{(Step 2/3e) performed" + "; make-methods-of-condition2xi-performed-by-setstate-of-Condition4Xi: " + str(self._make_methods) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:  
            message2print = "{(Step 2/3f) hit" + "; make-methods-of-condition2xi-hit-by-setstate-of-Condition4Xi: " + " *** " + str(self._make_methods)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
                                      
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 3/3) returning" + "; setstate-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 

    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "make-methods-of-Condition4Xi" usage = "makes methods from locks">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _make_methods(self):
        '''
        makes methods from locks
        '''
        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = '_make_methods'
        
        
        message2print = "{(Step 1/3) started" + "; make-methods-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # ----------------------------------------------------------------------------------------- 
        self.acquire = self._lock.acquire
        
        message2print = "{(Step 2/3a) set" + "; acquire-of-condition2xi-set-by-make-methods-of-Condition4Xi to: " + str(self.acquire)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
                        
        
        self.release = self._lock.release
        
        message2print = "{(Step 2/3b) set" + "; release-of-condition2xi-set-by-make-methods-of-Condition4Xi to: " + str(self.release)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
            
        
        self.__enter__ = self._lock.__enter__
        
        message2print = "{(Step 2/3c) set" + "; enter-of-condition2xi-set-by-make-methods-of-Condition4Xi to: " + str(self.__enter__)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
            
        
        self.__exit__ = self._lock.__exit__
        
        message2print = "{(Step 2/3d) set" + "; exit-of-condition2xi-set-by-make-methods-of-Condition4Xi to: " + str(self.__exit__)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
    
                       
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 3/3) returning" + "; make-methods-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                        
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "repr-of-Condition4Xi" usage = "proxies with (self)">                            
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __repr__(self):
        '''
        Condition4Xi method __repr__: proxies with (self)
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = '__repr__'
        
        
        message2print = "{(Step 1/3) started" + "; repr-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 2/3a) using" + "; sleeping-count-semlock-of-condition2xi-using-by-repr-of-Condition4Xi: " + str(self._sleeping_count._semlock._get_value())  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        message2print = "{(Step 2/3b) using" + "; woken-count-semlock-of-condition2xi-using-by-repr-of-Condition4Xi: " + str(self._woken_count._semlock._get_value())  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                        
        
        try:
        
            num_waiters2xi = (self._sleeping_count._semlock._get_value() -
                           self._woken_count._semlock._get_value())
            
            message2print = "{(Step 2/3c) calculated" + "; num-waiters2xi-calculated-by-repr-of-Condition4Xi: " + str(num_waiters2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
                           
                           
        except Exception:
            num_waiters2xi = 'unkown'
            
            message2print = "{(Step 2/3d) hit" + "; num-waiters2xi-hit-by-repr-of-Condition4Xi: " + " *** " + str(num_waiters2xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/3a) returning" + "; lock-of-condition2xi-returning-by-repr-of-Condition4Xi: " + str(self._lock) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                        
        message2print = "{(Step 3/3b) returning" + "; num-waiters2xi-returning-by-repr-of-Condition4Xi: " + str(num_waiters2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                
        return '<Condition(%s, %s)>' % (self._lock, num_waiters2xi)
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait-of-Condition4Xi" usage = "waits according to given timeout">                      
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def wait(self, timeout=None):
        '''
        waits according to given timeout
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/7) 
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = 'wait'
        
        message2print = "{(Step 1/7a) started" + "; wait-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                                
        timeout = omega2timeout4xi(class2print, method2print) 

        message2print = "{(Step 1/7b) set" + "; timeout-set-by-wait-of-Condition4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
                    
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/7) 
        # -----------------------------------------------------------------------------------------         
        assert self._lock._semlock._is_mine(), \
               'condition4xi: wait: must acquire() condition before using wait()'
        
        message2print = "{(Step 2/7) asserted" + "; lock-semlock-is-mine-of-condition2xi-asserted-by-wait-of-Condition4Xi: " + str(self._lock._semlock._is_mine) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/7) 
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/7a) informing" + "; sleep-informing-by-wait-of-Condition4Xi: " + " thread is going to sleep " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        try:
            self._sleeping_count.release()
            
            message2print = "{(Step 3/7b) performed" + "; sleeping-count-release-of-condition2xi-performed-by-wait-of-Condition4Xi: " + str(self._sleeping_count.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
        except:
            message2print = "{(Step 3/7c) hit" + "; sleeping-count-release-of-condition2xi-hit-by-wait-of-Condition4Xi: " + " *** " + str(self._sleeping_count.release)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
                          
            raise    
            
            
        # -----------------------------------------------------------------------------------------    
        # (Step 4/7) 
        # ----------------------------------------------------------------------------------------- 
        count2xicount2xi= self._lock._semlock._count()
        
        message2print = "{(Step 4/7a) set" + "; count2xi-set-by-wait-of-Condition4Xi to: " + str(count2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    

        for i2xi in range(count2xi):
            self._lock.release()
        
            message2print = "{(Step 4/7b) performed" + "; lock-release-of-condition2xi-performed-by-wait-of-Condition4Xi: " + str(self._lock.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                                
        # -----------------------------------------------------------------------------------------    
        #(Step 5/7)  
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 5/7a) passing" + "; True-passing-to-wait-semaphore-acquire-of-condition2xi-by-wait-of-Condition4Xi: " + str(True) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                 
        message2print = "{(Step 5/7b) passing" + "; timeout-passing-to-wait-semaphore-acquire-of-condition2xi-by-wait-of-Condition4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                       
        
        try:
            # -------------------------------------------------------------------------------------    
            # 
            # -------------------------------------------------------------------------------------                 

            self._wait_semaphore.acquire(True, timeout)
            
            message2print = "{(Step 5/7c) performed" + "; wait-semaphore-acquire-of-condition2xi-performed-by-wait-of-Condition4Xi: " + str(self._wait_semaphore.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                
            
                    
        finally:
            # -------------------------------------------------------------------------------------    
            # (Step 6/7)
            # -------------------------------------------------------------------------------------                 
            
            self._woken_count.release()

            message2print = "{(Step 6/7a) performed" + "; woken-count-release-of-condition2xi-performed-by-wait-of-Condition4Xi: " + str(self._woken_count.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            message2print = "{(Step 6/7b) informing" + "; thread-informing-by-wait-of-Condition4Xi: " + " thread has woken up " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  

            for index2xi in range(count2xi):
                self._lock.acquire()
                
                message2print = "{(Step 6/7c1) using" + "; index2xi-using-by-wait-of-Condition4Xi: " + str(index2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                message2print = "{(Step 6/7c2) performed" + "; lock-acquire-of-condition2xi-performed-by-wait-of-Condition4Xi: " + str(self._lock.acquire) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
 
        # -----------------------------------------------------------------------------------------    
        # (Step 7/7)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 7/7) returning" + "; wait-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                                
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "notify-of-Condition4Xi" usage = "notifies">                            
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def notify(self):
        '''
        notifies
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/5)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = 'notify'
        
        
        message2print = "{(Step 1/5a) started" + "; notify-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        assert self._lock._semlock._is_mine(), 'Condition4Xi method notify: lock is not owned'
        
        message2print = "{(Step 1/5b) asserted" + "; lock-semlock-is-mine-of-condition2xi-asserted-by-notify-of-Condition4Xi: " + str(self._lock._semlock._is_mine) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
      
        
        assert not self._wait_semaphore.acquire(False)

        
        message2print = "{(Step 1/5c) asserted" + "; wait-semaphore-acquire-of-condition2xi-asserted-by-notify-of-Condition4Xi: " + str(self._wait_semaphore.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
            
        # -----------------------------------------------------------------------------------------    
        # (Step 2/5)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 2/5a) informing" + "; while-loop-informing-by-notify-of-Condition4Xi: " + " taking into account timeouts since last notify " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/5)
        # -----------------------------------------------------------------------------------------
        while self._woken_count.acquire(False):
            res2xi = self._sleeping_count.acquire(False)
            
            message2print = "{(Step 3/5a) set" + "; res2xi-set-by-notify-of-Condition4Xi to: " + str(res2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

            assert res2xi

            message2print = "{(Step 3/5b) asserted" + "; res2xi-asserted-by-notify-of-Condition4Xi: " + str(res2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                          
        if self._sleeping_count.acquire(False):
         
            # -------------------------------------------------------------------------------------    
            #(Step 4/5)
            # -------------------------------------------------------------------------------------                 
       
            self._wait_semaphore.release()      

            message2print = "{(Step 4/5a) performed" + "; wait-semaphore-release-of-condition2xi-performed-by-notify-of-Condition4Xi: " + str(self._wait_semaphore.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                                        
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
 
            self._woken_count.acquire()         

            message2print = "{(Step 4/5a) performed" + "; woken-count-acquire-of-condition2xi-performed-by-notify-of-Condition4Xi: " + str(self._woken_count.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
               
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 

            self._wait_semaphore.acquire(False)


            message2print = "{(Step 4/5a) performed" + "; wait-semaphore-acquire-of-condition2xi-performed-by-notify-of-Condition4Xi: " + str(self._wait_semaphore.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                 
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 5/5)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{ (Step 5/5) returning" + "; notify-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                        
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "notify-all-of-Condition4Xi" usage = "notifies all">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def notify_all(self):
        '''
        notifies all
        '''
        # -----------------------------------------------------------------------------------------    
        #(Step 1/5)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = 'notify_all'
        
        
        message2print = "{(Step 1/5a) started" + "; notify-all-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        assert self._lock._semlock._is_mine(), 'Condition4Xi method notify_all: lock is not owned'

        message2print = "{(Step 1/5b) asserted" + "; lock-semlock-is-mine-of-condition2xi-asserted-by-notify-all-of-Condition4Xi: " + str(self._lock._semlock._is_mine) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        
        assert not self._wait_semaphore.acquire(False)
        

        message2print = "{(Step 1/5b) asserted" + "; wait-semaphore-acquire-of-condition2xi-asserted-by-notify-all-of-Condition4Xi: " + str(self._wait_semaphore.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
     
          
        
        # -----------------------------------------------------------------------------------------    
        #(Step 2/5)
        # ----------------------------------------------------------------------------------------- 


        while self._woken_count.acquire(False):
            res2xi = self._sleeping_count.acquire(False)
            
            message2print = "{(Step 2/5a) set" + "; res2xi-set-by-notify-all-of-Condition4Xi to: " + str(res2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
            assert res2xi

            message2print = "{(Step 2/5b) asserted" + "; res2xi-asserted-by-notify-all-of-Condition4Xi: " + str(res2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                     
        # -----------------------------------------------------------------------------------------    
        #(Step 3/5)
        # ----------------------------------------------------------------------------------------- 
        sleepers2xi = 0
        
        message2print = "{(Step 3/5a) set" + "; sleepers2xi-set-by-notify-all-of-Condition4Xi to: " + str(sleepers2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
                
                
        while self._sleeping_count.acquire(False):
            self._wait_semaphore.release()        # wake up one sleeper
            
            message2print = "{(Step 3/5b) performed" + "; wait-semaphore-release-of-condition2xi-performed-by-notify-all-of-Condition4Xi: " + str(self._wait_semaphore.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            sleepers2xi += 1
            
            message2print = "{(Step 3/5c) calculated" + "; sleepers2xi-calculated-by-notify-all-of-Condition4Xi: " + str(sleepers2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/5)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 4/5a) using" + "; sleepers2xi-using-by-notify-all-of-Condition4Xi: " + str(sleepers2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
        if sleepers2xi:
            for index2xi in range(sleepers2xi):
                self._woken_count.acquire()       # wait for a sleeper to wake
                
                message2print = "{(Step 4/5b) using" + "; index2x-using-by-notify-all-of-Condition4Xi: " + str(index2x)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 4/5c) performed" + "; woken-count-acquire-of-condition2xi-performed-by-notify-all-of-Condition4Xi: " + str(self._woken_count.acquire) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                  

            # rezero wait_semaphore in case some timeouts just happened
            while self._wait_semaphore.acquire(False):
                
                
                message2print = "{(Step 4/5d) waiting" + "; wait-semaphore-acquire-of-condition2xi-waiting-by-notify-all-of-Condition4Xi: " + str(self._wait_semaphore.acquire) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                     
            
                pass
                
                
        
        # -----------------------------------------------------------------------------------------    
        # (Step 5/5)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 5/5) returning" + "; notify-all-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
               
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait-for-of-Condition4Xi" usage = "notifies all">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def wait_for(self, predicate, timeout=None):
        '''
        notifies all
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/5)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = 'wait_for'
        
        
        message2print = "{(Step 1/5) started" + "; wait-for-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        result = predicate()
        
        if result:
            message2print = "{(Step 2/5) returning" + "; result-returning-by-wait-for-of-Condition4Xi: " + str(result) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            # 
        
            return result
        if timeout is not None:
            endtime = _time() + timeout
        else:
            endtime = None
            waittime = None
            
        while not result:
            if endtime is not None:
                waittime = endtime - _time()
                if waittime <= 0:
                    break
                    
            self.wait(waittime)
            
            result = predicate()
            
        message2print = "{(Step 3/3b) returning" + "; result-returning-by-wait-for-of-Condition4Xi: " + str(result) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # 
           
        return result

    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "enter-of-Condition4Xi" usage = "returns enter from locks">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __enter__(self):
        '''
        returns enter from locks
        '''
        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = '__enter__'
        
        
        message2print = "{(Step 1/3) started" + "; enter-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  

        # -----------------------------------------------------------------------------------------    
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 3/3) returning" + "; enter-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
        return self._lock.__enter__()        
                        
    #</methodize4xi>                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "exit-of-Condition4Xi" usage = "returns exit from locks">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __exit__(self):
        '''
        returns exit from locks
        '''
        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'Condition4Xi'
        method2print = '__exit__'
        
        
        message2print = "{(Step 1/3) started" + "; exit-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  

        # -----------------------------------------------------------------------------------------    
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 3/3) returning" + "; exit-of-Condition4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
        return self._lock.__exit__(*args)
                        
    #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Condition4Xi" usage = "senses Condition4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses Condition4Xi class objects
        '''
          
        
        class2print = 'Condition4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Condition4Xi-sensing-by-sense-of-Condition4Xi: " + str(Condition4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Condition4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['inheritance-of-condition2xi'] = object
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Condition4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
      
        message2print = "{sensed" + "; Condition4Xi-sensed-by-sense-of-Condition4Xi: " + str(Condition4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Condition4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Condition4Xi" usage = "reasons Condition4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons Condition4Xi class objects
        '''
                 
        
        class2print = 'Condition4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Condition4Xi-reasoning-by-reason-of-Condition4Xi: " + str(Condition4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-Condition4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                         
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; Condition4Xi-reasoned-by-reason-of-Condition4Xi: " + str(Condition4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Condition4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Condition4Xi" usage = "aces Condition4Xi class objects ">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces Condition4Xi class objects 
        '''
               
        
        
        class2print = 'Condition4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-Condition4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Condition4Xi-acing-by-ace-of-Condition4Xi: " + str(Condition4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{acquiring" + "; kb8xi-acquiring-by-ace-of-Condition4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                 
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; Condition4Xi-aced-by-ace-of-Condition4Xi: " + str(Condition4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{calling" + "; kb8xi-calling-by-ace-of-Condition4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
                        
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------     
    #</methodize4xi>
    
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of condition4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

